@@ -2,7 +2,7 @@
 # Apache License 2.0, Copyright 2019 Tebel.Automation Private Limited
 # https://github.com/tebelorg/RPA-Python/blob/master/LICENSE.txt
 __author__ = 'Ken Soh <opensource@tebel.org>'
-__version__ = '1.42.0'
+__version__ = '1.43.0'
 
 import subprocess
 import os
@@ -1230,7 +1230,7 @@ def ask(text_to_prompt = ''):
         else:
             return input(text_to_prompt + space_padding)
 
-def telegram(telegram_id = None, text_to_send = None):
+def telegram(telegram_id = None, text_to_send = None, custom_endpoint = None):
     if telegram_id is None or telegram_id == '':
         print('[RPA][ERROR] - Telegram ID missing for telegram()')
         return False
@@ -1245,17 +1245,25 @@ def telegram(telegram_id = None, text_to_send = None):
     telegram_endpoint = 'https://tebel.org/rpapybot'
     telegram_params = {'chat_id': telegram_id, 'text': text_to_send}
 
-    if _python2_env():
-        import json; import urllib
-        telegram_endpoint = telegram_endpoint + '/sendMessage.php?' + urllib.urlencode(telegram_params)
-        telegram_response = urllib.urlopen(telegram_endpoint).read()
-        return json.loads(telegram_response)['ok']
-        
-    else:
-        import json; import urllib.request; import urllib.parse
-        telegram_endpoint = telegram_endpoint + '/sendMessage.php?' + urllib.parse.urlencode(telegram_params)
-        telegram_response = urllib.request.urlopen(telegram_endpoint).read()
-        return json.loads(telegram_response)['ok']
+    if custom_endpoint is not None and custom_endpoint != '':
+        telegram_endpoint = custom_endpoint
+
+    # handle case where no internet or url is invalid
+    try:
+        if _python2_env():
+            import json; import urllib
+            telegram_endpoint = telegram_endpoint + '/sendMessage.php?' + urllib.urlencode(telegram_params)
+            telegram_response = urllib.urlopen(telegram_endpoint).read()
+            return json.loads(telegram_response)['ok']
+
+        else:
+            import json; import urllib.request; import urllib.parse
+            telegram_endpoint = telegram_endpoint + '/sendMessage.php?' + urllib.parse.urlencode(telegram_params)
+            telegram_response = urllib.request.urlopen(telegram_endpoint).read()
+            return json.loads(telegram_response)['ok']
+
+    except Exception as e:
+        return False
 
 def keyboard(keys_and_modifiers = None):
     if not _started():
@@ -1727,8 +1735,34 @@ def download_location(location = None):
         _tagui_download_directory = location
         return True
 
-def get_text(text = None, left = None, right = None, count = 1):
-    if text is None or left is None or right is None:
+def get_text(source_text = None, left = None, right = None, count = 1):
+    if source_text is None or left is None or right is None:
         return ''
 
-    return text
+    left_position = source_text.find(left)
+    if left_position == -1: return ''
+    right_position = source_text.find(right, left_position + 1)
+    if right_position == -1: return ''
+
+    if count > 1:
+        occurrence_count = 2
+        while occurrence_count <= count:
+            occurrence_count += 1
+            left_position = source_text.find(left, right_position + 1)
+            if left_position == -1: return ''
+            right_position = source_text.find(right, left_position + 1)
+            if right_position == -1: return ''
+
+    return source_text[left_position + len(left) : right_position].strip()
+
+def del_chars(source_text = None, characters = None):
+    if source_text is None:
+        return ''
+
+    elif characters is None:
+        return source_text
+
+    for character in characters:
+        source_text = source_text.replace(character, '')
+
+    return source_text
